//
//  SimplePlug.m
//  iSimplePlug
//
//  $RCSfile: SimplePlug.m,v $
//  $Author: paploo $
//  $Revision: 1.2 $
//  $Date: 2003/12/13 07:14:56 $
//  $Name:  $
//  $Source: /cvsroot/itonamaton/plugins/iSimplePlug/SimplePlug.m,v $
//
//  Created by Jeffrey Reinecke on Wed Dec 10 2003.
//  This source is part of the iTonamaton project, and is released under
//  the GNU GPL.  The iTonamaton project can be found at
//  http://itonamaton.sf.net/
//

/*******************************************************************************
 * This source file is meant to go along with the tutorial that is in
 * the Documentation folder in the project directory.  The source in this
 * file is necessarily sparsly commented so that it is easier to see how all the
 * source fits together.  However, some comments were inserted to help guide
 * those who have not read the tutorial document, or those who need memory
 * refreshing, as to what the code does.
 ******************************************************************************/

//Import the header file that defines the SimplePlug class.
#import "SimplePlug.h"


@implementation SimplePlug

/*
 * Override the defailt init method.  Note that this init method should never
 * be called by iTonamaton.
 */
- (id)init
{
    return [self initWithOwner:nil];
}

/*
 * This is the init method that is called by iTonamaton.  The passed owner
 * object is passed off to our super class for storage.
 */
- (id)initWithOwner:(ITonamaton *)ownerObj
{
    //init our super class and assign the returned object to self.  If it is
    //not nil, initialize self.
    if( self = [super initWithOwner:ownerObj] )
    {
        //Initialize the commandNameArray.
        commandNameArray = [[NSArray alloc] initWithObjects:@"monkey", @"cow", nil];
        
        //Allocate the commandID Array.
        commandIDArray = calloc( [commandNameArray count], sizeof(unsigned) );
        //If the allocation is unsuccessful, return nil.
        if( !commandIDArray )
        {
            [self release];
            return nil;
        }
    }
    //Return self
    return self;
}

/*
 * This method is called on us automatically, after registration of the plugin
 * is complete.
 */
- (void)pluginRegistered
{
    // Cache the command IDs in the commandIDArray.
    unsigned i;
    for( i=0; i<[commandNameArray count]; i++ )
        commandIDArray[i] = [self identifierForCommand:[commandNameArray objectAtIndex:i]];
    
    //Register with the event notification center to listen for command received
    //events.  The processCommand: method is notified whent these events occur.
    [[[self owner] eventCenter]
            addObserver:self
            selector:@selector(processCommand:)
            name:ITCommandReceivedNotification
            object:[self owner]];
}


/*
 * This method returns an NSArray of all the commands that this plugin can
 * respond to.
 */
- (NSArray *)commands
{
    return commandNameArray;
}

/*
 * This method returns a user readable string containing the name of the plugin.
 */
- (NSString *)pluginName
{
    return @"iSimplePlugin";
}

/*
 * This method returns a user readable string containing the version information
 * for this plugin.
 */
- (NSString *)pluginVersionString
{
    return @"CVS Revision $Revision: 1.2 $";
}

/*
 * This method was registered with the event notification cetner to recieve all
 * notifications of commands.  It then looks at the notification to see what
 * command was issued, and then processes it.
 */
- (void)processCommand:(NSNotification *)notification
{
    //Get the command ID from the notification.
    unsigned commandID = [self commandFromNotification:notification];
    
    
    //Determine if the command has the "help" argument associated with it.
    BOOL outputHelp = NO;
    NSArray * arguments = [[notification userInfo] objectForKey:ITCommandArgumentsKey];
    if( [arguments count] > 0 )
    {
        if( [[arguments objectAtIndex:0] caseInsensitiveCompare:@"help"] == NSOrderedSame )
        {
            outputHelp = YES;
        }
    }
    
    //Execute the code for the proper command, based off of the cached IDs.
    if( commandID == commandIDArray[0] )
    {
        if( outputHelp )
            [self sendMessage:@"!monkey outputs the sound a monkey makes." withNotification:notification];
        else
            [self sendMessage:@"Eep!" withNotification:notification];
    }
    else if( commandID == commandIDArray[1] )
    {
        if( outputHelp )
            [self sendMessage:@"!cow outputs the sound a cow makes." withNotification:notification];
        else
            [self sendMessage:@"Moo!" withNotification:notification];
    }
    else
    {
        NSLog( @"iSimplePlug: Command ID %u not recognized", commandID );
    }
}


/*
 * This method is called when the class is deallocated.  It releases all the
 * memory which it has used.
 */
- (void)dealloc
{
    [commandNameArray release];
    free( commandIDArray );
    
    [super dealloc];
}

@end
